#
# See the file "LICENSE" for the full license governing this code.
#
# frozen_string_literal: true
#
# This is a hiera lookup backend that you can use to fetch data based on locale puppet
# variables without hiera giving you a warning when the variable doesn't exist. It can only 
# be used in the context of a module at this point in time.
#
# You have to specify the `file_name` option in the `hiera.yaml`. Puppet variables
# can be used in this entry just like in regular puppet interpollation. Here is an example:
#
# ```yaml
#    - name: "Control data"
#    data_hash:  easy_type::yaml_data_without_warnings
#    options:
#    file_name: "benchmarks/cis/${db_version}/${doc_version}/control_data.yaml"
# ```
#
require 'yaml'
Puppet::Functions.create_function('easy_type::yaml_data_without_warnings') do

  dispatch :yaml_data_without_warnings do
    param 'Hash', :options
    param 'Any', :context
  end
  
  argument_mismatch :missing_path do
    param 'Hash', :options
    param 'Puppet::LookupContext', :context
  end
    
  def yaml_data_without_warnings(options, context)
    file_name = options['file_name']
    context.explain { "File specified in hiera.yaml is '#{file_name}'" }
    variable_values = {}
    variable_names = file_name.scan(/\$\{(.+?)\}/)&.flatten
    variable_names.each do |variable_name| 
      variable_values[variable_name] = fetch_variable(context, variable_name)
      if variable_values[variable_name].nil?
        context.explain {"Variable '#{variable_name}' not set. Skipping this lookup." }
        return context.not_found
      end
    end
    variable_names.each do |variable_name|
      file_name.gsub!("${#{variable_name}}", variable_values[variable_name])
    end
    context.explain { "File name after variable substitution is '#{file_name}'" }
    context.explain { "Running outside of module scope. skpping lookup." } if context.module_name.to_s == ''
    data_dir = File.expand_path(File.dirname(__FILE__) + "/../../../../../#{context.module_name}/data")
    path = "#{data_dir}/#{file_name}"
    if !File.exists?(path)
      context.explain { "File '#{path}' not found skipping this lookup." }
      return context.not_found
    end
    context.cached_file_data(path) do |content|
      begin
        #
        # Try to use the safest possible way to load the yaml. This depends on 
        # the used ruby versions.
        #
        if defined?(YAML.safe_load)
          data = YAML.safe_load(content, [Symbol], [], false, path)
        else
          data = YAML.load(content)
        end
    
        if data.is_a?(Hash)
          Puppet::Pops::Lookup::HieraConfig.symkeys_to_string(data)
        else
          msg = _("%{path}: file does not contain a valid yaml hash" % { path: path })
          raise Puppet::DataBinding::LookupError, msg if Puppet[:strict] == :error && data != false
          Puppet.warning(msg)
          {}
        end
      rescue Puppet::Util::Yaml::YamlLoadError => ex
        # YamlLoadErrors include the absolute path to the file, so no need to add that
        raise Puppet::DataBinding::LookupError, _("Unable to parse %{message}") % { message: ex.message }
      end
    end
  end

  def fetch_variable(context, name)
    catch(:undefined_variable) {
      return context.interpolate("%{#{name}}")
    }
    nil
  end

  def missing_path(options, context)
    "one of 'path', 'paths' 'glob', 'globs' or 'mapped_paths' must be declared in hiera.yaml when using this data_hash function"
  end
end